from typing import Self

from pydantic import BaseModel, Field

from line_works.enums.message_type import MessageType
from line_works.models.caller import Caller
from line_works.models.sticker import Sticker


class SendMessageRequest(BaseModel):
    service_id: str = Field(alias="serviceId", default="works")
    channel_no: int = Field(alias="channelNo")
    temp_message_id: int = Field(alias="tempMessageId", default=733428260)
    caller: Caller
    extras: str = Field(default="")
    content: str = Field(default="")
    type: MessageType

    class Config:
        populate_by_name = True

    @classmethod
    def text_message(cls, caller: Caller, channel_no: int, text: str) -> Self:
        return cls(
            channel_no=channel_no,
            content=text,
            caller=caller,
            type=MessageType.TEXT,
        )

    @classmethod
    def sticker_message(
        cls, caller: Caller, channel_no: int, sticker: Sticker
    ) -> Self:
        return cls(
            channel_no=channel_no,
            caller=caller,
            extras=sticker.model_dump_json(by_alias=True),
            type=MessageType.STICKER,
        )



from urllib.parse import urljoin


class TalkURL:
    HOST = "https://talk.worksmobile.com"

    MY_INFO = urljoin(HOST, "/p/contact/v3/domain/contacts/my")

    SEND_MESSAGE = urljoin(HOST, "/p/oneapp/client/chat/sendMessage")



from pydantic import BaseModel, Field


class Sticker(BaseModel):
    sticker_type: str = Field(alias="stkType")
    package_version: str = Field(alias="pkgVer")
    package_id: str = Field(alias="pkgId")
    sticker_id: str = Field(alias="stkId")
    sticker_option: str = Field(alias="stkOpt")

    class Config:
        populate_by_name = True



class LineWorksException(Exception):
    pass


class LogicException(LineWorksException):
    pass


class LoginException(LineWorksException):
    pass


class GetMyInfoException(LineWorksException):
    pass


class SendMessageException(LineWorksException):
    pass



from pydantic import BaseModel, Field


class SentMessage(BaseModel):
    message_id: int = Field(alias="messageId")
    channel_no: int = Field(alias="channelNo")
    writer_id: str = Field(alias="writerId")
    user_no: int = Field(alias="userNo")
    bot_no: int = Field(alias="botNo")
    message_no: int = Field(alias="messageNo")
    content: str
    member_count: int = Field(alias="memberCount")
    message_type_code: int = Field(alias="messageTypeCode")
    message_status_type: str = Field(alias="messageStatusType")
    message_status_type_code: int = Field(alias="messageStatusTypeCode")
    extras: str = Field(default="")
    tid: int
    create_time: int = Field(alias="createTime")
    update_time: int = Field(alias="updateTime")

    class Config:
        populate_by_name = True



from pydantic import BaseModel, Field


class Caller(BaseModel):
    domain_id: int = Field(alias="domainId")
    user_no: int = Field(alias="userNo")

    class Config:
        populate_by_name = True



from pydantic import BaseModel, Field


class Name(BaseModel):
    first_name: str = Field(alias="firstName")
    last_name: str = Field(alias="lastName")
    phonetic_first_name: str = Field(alias="phoneticFirstName")
    phonetic_last_name: str = Field(alias="phoneticLastName")
    phonetic_name: str = Field(alias="phoneticName")
    display_name: str = Field(alias="displayName")

    class Config:
        populate_by_name = True


class Organization(BaseModel):
    domain_id: int = Field(alias="domainId")
    organization: str
    groups: list[str]

    class Config:
        populate_by_name = True


class Email(BaseModel):
    content: str
    type_code: str = Field(alias="typeCode")
    represent: bool

    class Config:
        populate_by_name = True


class UserService(BaseModel):
    service_type: str = Field(alias="serviceType")
    works_at_user_no: int = Field(alias="worksAtUserNo")
    name: str
    id: str
    invite_url: str = Field(alias="inviteUrl")

    class Config:
        populate_by_name = True


class WorksAt(BaseModel):
    id: str
    invite_url: str = Field(alias="inviteUrl")
    users: list[UserService]
    works_at_count: int = Field(alias="worksAtCount")
    id_search_block: bool = Field(alias="idSearchBlock")

    class Config:
        populate_by_name = True


class GetMyInfoResponse(BaseModel):
    tenant_id: int = Field(alias="tenantId")
    domain_id: int = Field(alias="domainId")
    contact_no: int = Field(alias="contactNo")
    read_only: bool = Field(alias="readOnly")
    temp_id: bool = Field(alias="tempId")
    name: Name
    i18n_name: str = Field(alias="i18nName")
    i18n_names: list[str] = Field(alias="i18nNames")
    photos: list[str]
    organizations: list[Organization]
    emails: list[Email]
    telephones: list[str]
    messengers: list[str]
    position: str
    department: str
    location: str
    important: bool
    executive: bool
    photo_hash: str = Field(alias="photoHash")
    works_at: WorksAt = Field(alias="worksAt")
    access_limit: bool = Field(alias="accessLimit")
    user_photo_modify: bool = Field(alias="userPhotoModify")
    user_absence_modify: bool = Field(alias="userAbsenceModify")
    organization: str
    groups: list[str]
    works_services: list[str] = Field(alias="worksServices")
    custom_fields: list[str] = Field(alias="customFields")
    profile_statuses: list[str] = Field(alias="profileStatuses")
    profile_statuses_v2: list[str] = Field(alias="profileStatusesV2")
    instance: int

    class Config:
        populate_by_name = True



import json
from os import makedirs
from os.path import exists
from os.path import join as path_join
from typing import Any, Type

from pydantic import BaseModel, Field, PrivateAttr
from requests import HTTPError, JSONDecodeError, Session

from line_works import config
from line_works.constants import RequestType
from line_works.decorator import save_cookie
from line_works.enums.yes_no_option import YesNoOption
from line_works.exceptions import (
    GetMyInfoException,
    LoginException,
    SendMessageException,
)
from line_works.logger import get_file_path_logger
from line_works.models.caller import Caller
from line_works.models.sticker import Sticker
from line_works.requests.login import LoginRequest
from line_works.requests.send_message import SendMessageRequest
from line_works.responses.get_my_info import GetMyInfoResponse
from line_works.responses.send_message import SendMessageResponse
from line_works.urls.auth import AuthURL
from line_works.urls.talk import TalkURL

logger = get_file_path_logger(__name__)


class LineWorks(BaseModel):
    works_id: str
    password: str = Field(repr=False)
    keep_login: YesNoOption = Field(repr=False, default=YesNoOption.YES)
    remember_id: YesNoOption = Field(repr=False, default=YesNoOption.YES)
    tenant_id: int = Field(init=False, default=0)
    domain_id: int = Field(init=False, default=0)
    contact_no: int = Field(init=False, default=0)
    session: Session = Field(init=False, repr=False, default_factory=Session)
    _caller: Caller = PrivateAttr()

    class Config:
        arbitrary_types_allowed = True

    @property
    def session_dir(self) -> str:
        return path_join(config.SESSION_DIR, self.works_id)

    @property
    def cookie_path(self) -> str:
        return path_join(self.session_dir, "cookie.json")

    def model_post_init(self, __context: Any) -> None:
        makedirs(self.session_dir, exist_ok=True)
        self.session.headers.update(config.HEADERS)

        if exists(self.cookie_path):
            # login with cookie
            with open(self.cookie_path) as j:
                c = json.load(j)
            self.session.cookies.update(c)

        try:
            my_info = self.get_my_info()
        except Exception:
            self.session.cookies.clear()
            self.login_with_id()

        my_info = self.get_my_info()
        self.tenant_id = my_info.tenant_id
        self.domain_id = my_info.domain_id
        self.contact_no = my_info.contact_no
        self._caller = Caller(
            domain_id=self.domain_id, user_no=self.contact_no
        )

        logger.info(f"login success: {self!r}")

    def _request_with_error_handling(
        self,
        method: str,
        url: str,
        ex: Type[Exception],
        **kwargs: Any,
    ) -> dict[str, Any]:
        try:
            r = self.session.request(method, url, **kwargs)
            r.raise_for_status()
            return r.json()  # type: ignore
        except HTTPError as e:
            raise ex(f"HTTP error: {e}") from e
        except JSONDecodeError as e:
            raise ex(f"Invalid response: [{r.status_code}] {r.url}") from e
        except Exception as e:
            raise ex(f"Unexpected error: {e}") from e

    @save_cookie
    def login_with_id(self) -> None:
        self.session.get(AuthURL.LOGIN)

        try:
            r = self.session.post(
                AuthURL.LOGIN_PROCESS_V2,
                data=LoginRequest(
                    input_id=self.works_id,
                    password=self.password,
                    keep_login=self.keep_login,
                    remember_id=self.remember_id,
                ).model_dump(by_alias=True),
            )
            r.raise_for_status()
        except HTTPError as e:
            raise LoginException(e)

    def get_my_info(self) -> GetMyInfoResponse:
        d = self._request_with_error_handling(
            RequestType.GET,
            TalkURL.MY_INFO,
            GetMyInfoException,
        )
        return GetMyInfoResponse.model_validate(d)  # type: ignore

    def send_message(self, req: SendMessageRequest) -> SendMessageResponse:
        d = self._request_with_error_handling(
            RequestType.POST,
            TalkURL.SEND_MESSAGE,
            SendMessageException,
            json=req.model_dump(by_alias=True),
        )
        return SendMessageResponse.model_validate(d)  # type: ignore

    def send_text_message(self, to: int, text: str) -> SendMessageResponse:
        return self.send_message(
            SendMessageRequest.text_message(self._caller, to, text)
        )

    def send_sticker_message(
        self, to: int, sticker: Sticker
    ) -> SendMessageResponse:
        return self.send_message(
            SendMessageRequest.sticker_message(self._caller, to, sticker)
        )
